@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2012 David Robillard <http://drobilla.net>
+  Copyright 2011-2015 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -21,6 +21,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define SERDI_ERROR(msg)       fprintf(stderr, "serdi: " msg);
+#define SERDI_ERRORF(fmt, ...) fprintf(stderr, "serdi: " fmt, __VA_ARGS__);
+
 typedef struct {
 	SerdEnv*    env;
 	SerdWriter* writer;
@@ -30,7 +33,7 @@ static int
 print_version(void)
 {
 	printf("serdi " SERD_VERSION " <http://drobilla.net/software/serd>\n");
-	printf("Copyright 2011-2012 David Robillard <http://drobilla.net>.\n"
+	printf("Copyright 2011-2013 David Robillard <http://drobilla.net>.\n"
 	       "License: <http://www.opensource.org/licenses/isc>\n"
 	       "This is free software; you are free to change and redistribute it."
 	       "\nThere is NO WARRANTY, to the extent permitted by law.\n");
@@ -41,6 +44,7 @@ static int
 print_usage(const char* name, bool error)
 {
 	FILE* const os = error ? stderr : stdout;
+	fprintf(os, "%s", error ? "\n" : "");
 	fprintf(os, "Usage: %s [OPTION]... INPUT [BASE_URI]\n", name);
 	fprintf(os, "Read and write RDF syntax.\n");
 	fprintf(os, "Use - for INPUT to read from standard input.\n\n");
@@ -50,6 +54,7 @@ print_usage(const char* name, bool error)
 	fprintf(os, "  -f           Keep full URIs in input (don't qualify).\n");
 	fprintf(os, "  -h           Display this help and exit.\n");
 	fprintf(os, "  -i SYNTAX    Input syntax (`turtle' or `ntriples').\n");
+	fprintf(os, "  -l           Lax (non-strict) parsing.\n");
 	fprintf(os, "  -o SYNTAX    Output syntax (`turtle' or `ntriples').\n");
 	fprintf(os, "  -p PREFIX    Add PREFIX to blank node IDs.\n");
 	fprintf(os, "  -q           Suppress all output except data.\n");
@@ -67,17 +72,17 @@ set_syntax(SerdSyntax* syntax, const char* name)
 	} else if (!strcmp(name, "ntriples")) {
 		*syntax = SERD_NTRIPLES;
 	} else {
-		fprintf(stderr, "Unknown input format `%s'\n", name);
+		SERDI_ERRORF("unknown syntax `%s'\n", name);
 		return false;
 	}
 	return true;
 }
 
 static int
-bad_arg(const char* name, char opt)
+missing_arg(const char* name, char opt)
 {
-	fprintf(stderr, "%s: Bad or missing value for -%c\n", name, opt);
-	return 1;
+	SERDI_ERRORF("option requires an argument -- '%c'\n", opt);
+	return print_usage(name, true);
 }
 
 static SerdStatus
@@ -100,6 +105,7 @@ main(int argc, char** argv)
 	bool           bulk_read     = true;
 	bool           bulk_write    = false;
 	bool           full_uris     = false;
+	bool           lax           = false;
 	bool           quiet         = false;
 	const uint8_t* in_name       = NULL;
 	const uint8_t* add_prefix    = NULL;
@@ -119,6 +125,8 @@ main(int argc, char** argv)
 			full_uris = true;
 		} else if (argv[a][1] == 'h') {
 			return print_usage(argv[0], false);
+		} else if (argv[a][1] == 'l') {
+			lax = true;
 		} else if (argv[a][1] == 'q') {
 			quiet = true;
 		} else if (argv[a][1] == 'v') {
@@ -129,36 +137,40 @@ main(int argc, char** argv)
 			++a;
 			break;
 		} else if (argv[a][1] == 'i') {
-			if (++a == argc || !set_syntax(&input_syntax, argv[a])) {
-				return bad_arg(argv[0], 'i');
+			if (++a == argc) {
+				return missing_arg(argv[0], 'i');
+			} else if (!set_syntax(&input_syntax, argv[a])) {
+				return print_usage(argv[0], true);
 			}
 		} else if (argv[a][1] == 'o') {
-			if (++a == argc || !set_syntax(&output_syntax, argv[a])) {
-				return bad_arg(argv[0], 'o');
+			if (++a == argc) {
+				return missing_arg(argv[0], 'o');
+			} else if (!set_syntax(&output_syntax, argv[a])) {
+				return print_usage(argv[0], true);
 			}
 		} else if (argv[a][1] == 'p') {
 			if (++a == argc) {
-				return bad_arg(argv[0], 'p');
+				return missing_arg(argv[0], 'p');
 			}
 			add_prefix = (const uint8_t*)argv[a];
 		} else if (argv[a][1] == 'c') {
 			if (++a == argc) {
-				return bad_arg(argv[0], 'c');
+				return missing_arg(argv[0], 'c');
 			}
 			chop_prefix = (const uint8_t*)argv[a];
 		} else if (argv[a][1] == 'r') {
 			if (++a == argc) {
-				return bad_arg(argv[0], 'r');
+				return missing_arg(argv[0], 'r');
 			}
 			root_uri = (const uint8_t*)argv[a];
 		} else {
-			fprintf(stderr, "%s: Unknown option `%s'\n", argv[0], argv[a]);
+			SERDI_ERRORF("invalid option -- '%s'\n", argv[a] + 1);
 			return print_usage(argv[0], true);
 		}
 	}
 
 	if (a == argc) {
-		fprintf(stderr, "%s: Missing input\n", argv[0]);
+		SERDI_ERROR("missing input\n");
 		return 1;
 	}
 
@@ -195,9 +207,9 @@ main(int argc, char** argv)
 		}
 	}
 
-	if (input_syntax != SERD_NTRIPLES  // Base URI may change (@base)
-	    || (output_syntax == SERD_TURTLE)) {
-		output_style |= SERD_STYLE_RESOLVED;
+	if (input_syntax != SERD_NTRIPLES || (output_style & SERD_STYLE_CURIED)) {
+		// Base URI may change and/or we're abbreviating URIs, so must resolve
+		output_style |= SERD_STYLE_RESOLVED;  // Base may chan
 	}
 
 	if (bulk_write) {
@@ -215,6 +227,7 @@ main(int argc, char** argv)
 		(SerdStatementSink)serd_writer_write_statement,
 		(SerdEndSink)serd_writer_end_anon);
 
+	serd_reader_set_strict(reader, !lax);
 	if (quiet) {
 		serd_reader_set_error_sink(reader, quiet_error_sink, NULL);
 		serd_writer_set_error_sink(writer, quiet_error_sink, NULL);
