@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2012 David Robillard <http://drobilla.net>
+  Copyright 2011-2015 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -97,7 +97,6 @@ struct SerdWriterImpl {
 };
 
 typedef enum {
-	WRITE_URI,
 	WRITE_STRING,
 	WRITE_LONG_STRING
 } TextContext;
@@ -105,6 +104,12 @@ typedef enum {
 static void
 w_err(SerdWriter* writer, SerdStatus st, const char* fmt, ...)
 {
+	/* TODO: This results in errors with no file information, which is not
+	   helpful when re-serializing a file (particularly for "undefined
+	   namespace prefix" errors.  The statement sink API needs to be changed to
+	   add a Cursor parameter so the source can notify the writer of the
+	   statement origin for better error reporting. */
+
 	va_list args;
 	va_start(args, fmt);
 	const SerdError e = { st, NULL, 0, 0, fmt, &args };
@@ -138,118 +143,235 @@ copy_node(SerdNode* dst, const SerdNode* src)
 static inline size_t
 sink(const void* buf, size_t len, SerdWriter* writer)
 {
-	if (writer->style & SERD_STYLE_BULK) {
+	if (len == 0) {
+		return 0;
+	} else if (writer->style & SERD_STYLE_BULK) {
 		return serd_bulk_sink_write(buf, len, &writer->bulk_sink);
 	} else {
 		return writer->sink(buf, len, writer->stream);
 	}
 }
 
+// Parse a UTF-8 character, set *size to the length, and return the code point
+static inline uint32_t
+parse_utf8_char(SerdWriter* writer, const uint8_t* utf8, size_t* size)
+{
+	uint32_t c = 0;
+	if ((utf8[0] & 0x80) == 0) {  // Starts with `0'
+		*size = 1;
+		c     = utf8[0];
+	} else if ((utf8[0] & 0xE0) == 0xC0) {  // Starts with `110'
+		*size = 2;
+		c     = utf8[0] & 0x1F;
+	} else if ((utf8[0] & 0xF0) == 0xE0) {  // Starts with `1110'
+		*size = 3;
+		c     = utf8[0] & 0x0F;
+	} else if ((utf8[0] & 0xF8) == 0xF0) {  // Starts with `11110'
+		*size = 4;
+		c     = utf8[0] & 0x07;
+	} else {
+		w_err(writer, SERD_ERR_BAD_ARG, "invalid UTF-8: %X\n", utf8[0]);
+		*size = 0;
+		return 0;
+	}
+
+	size_t  i  = 0;
+	uint8_t in = utf8[i++];
+
+#define READ_BYTE() \
+	in = utf8[i++] & 0x3F; \
+	c  = (c << 6) | in;
+
+	switch (*size) {
+	case 4: READ_BYTE();
+	case 3: READ_BYTE();
+	case 2: READ_BYTE();
+	}
+
+	return c;
+}
+
+// Write a single character, as an escape for single byte characters
+// (Caller prints any single byte characters that don't need escaping)
+static size_t
+write_character(SerdWriter* writer, const uint8_t* utf8, size_t* size)
+{
+	const uint8_t replacement_char[] = { 0xEF, 0xBF, 0xBD };
+	char          escape[11]         = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+	const uint8_t in                 = utf8[0];
+
+	uint32_t c = parse_utf8_char(writer, utf8, size);
+	switch (*size) {
+	case 0:
+		w_err(writer, SERD_ERR_BAD_ARG, "invalid UTF-8: %X\n", in);
+		return sink(replacement_char, sizeof(replacement_char), writer);
+	case 1:
+		snprintf(escape, sizeof(escape), "\\u%04X", in);
+		return sink(escape, 6, writer);
+	default:
+		break;
+	}
+
+	if (!(writer->style & SERD_STYLE_ASCII)) {
+		// Write UTF-8 character directly to UTF-8 output
+		return sink(utf8, *size, writer);
+	}
+
+	if (c <= 0xFFFF) {
+		snprintf(escape, sizeof(escape), "\\u%04X", c);
+		return sink(escape, 6, writer);
+	} else {
+		snprintf(escape, sizeof(escape), "\\U%08X", c);
+		return sink(escape, 10, writer);
+	}
+}
+
+static inline bool
+uri_must_escape(const uint8_t c)
+{
+	switch (c) {
+	case ' ': case '"': case '<': case '>': case '\\':
+	case '^': case '`': case '{': case '|': case '}':
+		return true;
+	default:
+		return !in_range(c, 0x20, 0x7E);
+	}
+}
+
+static size_t
+write_uri(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
+{
+	size_t len = 0;
+	for (size_t i = 0; i < n_bytes;) {
+		size_t j = i;  // Index of next character that must be escaped
+		for (; j < n_bytes; ++j) {
+			if (uri_must_escape(utf8[j])) {
+				break;
+			}
+		}
+
+		// Bulk write all characters up to this special one
+		len += sink(&utf8[i], j - i, writer);
+ 		if ((i = j) == n_bytes) {
+			break;  // Reached end
+		}
+
+		// Write UTF-8 character
+		size_t size = 0;
+		len += write_character(writer, utf8 + i, &size);
+		i   += size;
+	}
+	return len;
+}
+
+static bool
+lname_must_escape(const uint8_t c)
+{
+	/* This arbitrary list of characters, most of which have nothing to do with
+	   Turtle, must be handled as special cases here because the RDF and SPARQL
+	   WGs are apparently intent on making the once elegant Turtle a baroque
+	   and inconsistent mess, throwing elegance and extensibility completely
+	   out the window for no good reason.
+
+	   Note '-', '.', and '_' are also in PN_LOCAL_ESC, but are valid unescaped
+	   in local names, so they are not escaped here. */
+
+	switch (c) {
+	case '\'': case '!': case '#': case '$': case '%': case '&':
+	case '(': case ')': case '*': case '+': case ',': case '/':
+	case ';': case '=': case '?': case '@': case '~':
+		return true;
+	}
+	return false;
+}
+
+static size_t
+write_lname(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
+{
+	size_t len = 0;
+	for (size_t i = 0; i < n_bytes; ++i) {
+		size_t j = i;  // Index of next character that must be escaped
+		for (; j < n_bytes; ++j) {
+			if (lname_must_escape(utf8[j])) {
+				break;
+			}
+		}
+
+		// Bulk write all characters up to this special one
+		len += sink(&utf8[i], j - i, writer);
+ 		if ((i = j) == n_bytes) {
+			break;  // Reached end
+		}
+
+		// Write escape
+		len += sink("\\", 1, writer);
+		len += sink(&utf8[i], 1, writer);
+	}
+	return len;
+}
+
 static size_t
 write_text(SerdWriter* writer, TextContext ctx,
            const uint8_t* utf8, size_t n_bytes)
 {
-	size_t len        = 0;
-	char   escape[11] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+	size_t len = 0;
 	for (size_t i = 0; i < n_bytes;) {
 		// Fast bulk write for long strings of printable ASCII
 		size_t j = i;
 		for (; j < n_bytes; ++j) {
-			if (utf8[j] == '>' || utf8[j] == '\\' || utf8[j] == '"'
+			if (utf8[j] == '\\' || utf8[j] == '"'
 			    || (!in_range(utf8[j], 0x20, 0x7E))) {
 				break;
 			}
 		}
 
-		if (j > i) {
-			len += sink(&utf8[i], j - i, writer);
-			i = j;
-			continue;
+		len += sink(&utf8[i], j - i, writer);
+		if ((i = j) == n_bytes) {
+			break;  // Reached end
 		}
 
 		uint8_t in = utf8[i++];
 		if (ctx == WRITE_LONG_STRING) {
-			if (in == '\\') {
-				len += sink("\\\\", 2, writer); continue;
-			} else if (in == '\"' && i == n_bytes) {
-				len += sink("\\\"", 2, writer); continue;  // '"' at string end
+			switch (in) {
+			case '\\': len += sink("\\\\", 2, writer); continue;
+			case '\b': len += sink("\\b", 2, writer);  continue;
+			case '\n': case '\r': case '\t': case '\f':
+				len += sink(&in, 1, writer);  // Write character as-is
+				continue;
+			case '\"':
+				if (i == n_bytes) {  // '"' at string end
+					len += sink("\\\"", 2, writer);
+				} else {
+					len += sink(&in, 1, writer);
+				}
+				continue;
+			default: break;
 			}
-		} else {
+		} else if (ctx == WRITE_STRING) {
 			switch (in) {
 			case '\\': len += sink("\\\\", 2, writer); continue;
 			case '\n': len += sink("\\n", 2, writer);  continue;
 			case '\r': len += sink("\\r", 2, writer);  continue;
 			case '\t': len += sink("\\t", 2, writer);  continue;
-			case '"':
-				if (ctx == WRITE_STRING) {
-					len += sink("\\\"", 2, writer);
-					continue;
-				}  // else fall-through
+			case '"':  len += sink("\\\"", 2, writer); continue;
 			default: break;
 			}
-
-			if ((ctx == WRITE_STRING && in == '"') ||
-			    (ctx == WRITE_URI    && in == '>')) {
-				snprintf(escape, sizeof(escape), "\\u%04X",
-				         ctx == WRITE_STRING ? '"' : '>');
-				len += sink(escape, 6, writer);
-				continue;
+			if (writer->syntax != SERD_NTRIPLES) {
+				switch (in) {
+				case '\b': len += sink("\\b", 2, writer); continue;
+				case '\f': len += sink("\\f", 2, writer); continue;
+				}
 			}
 		}
 
-		uint32_t c    = 0;
-		size_t   size = 0;
-		if ((in & 0x80) == 0) {  // Starts with `0'
-			c = in & 0x7F;
-			if (in_range(c, 0x20, 0x7E)
-			    || (is_space(c) && ctx == WRITE_LONG_STRING)) {
-				len += sink(&in, 1, writer);  // Print ASCII character
-			} else {
-				snprintf(escape, sizeof(escape), "\\u%04X", c);
-				len += sink(escape, 6, writer);  // ASCII control character
-			}
-			continue;
-		} else if ((in & 0xE0) == 0xC0) {  // Starts with `110'
-			size = 2;
-			c    = in & 0x1F;
-		} else if ((in & 0xF0) == 0xE0) {  // Starts with `1110'
-			size = 3;
-			c    = in & 0x0F;
-		} else if ((in & 0xF8) == 0xF0) {  // Starts with `11110'
-			size = 4;
-			c    = in & 0x07;
-		} else {
-			w_err(writer, SERD_ERR_BAD_ARG, "invalid UTF-8: %X\n", in);
-			const uint8_t replacement_char[] = { 0xEF, 0xBF, 0xBD };
-			len += sink(replacement_char, sizeof(replacement_char), writer);
+		size_t size = 0;
+		len += write_character(writer, utf8 + i - 1, &size);
+
+		if (size == 0) {
 			return len;
 		}
 
-		if (ctx != WRITE_URI && !(writer->style & SERD_STYLE_ASCII)) {
-			// Write UTF-8 character directly to UTF-8 output
-			// TODO: Always parse and validate character?
-			len += sink(utf8 + i - 1, size, writer);
-			i += size - 1;
-			continue;
-		}
-
-#define READ_BYTE() \
-		in = utf8[i++] & 0x3f; \
-		c  = (c << 6) | in;
-
-		switch (size) {
-		case 4: READ_BYTE();
-		case 3: READ_BYTE();
-		case 2: READ_BYTE();
-		}
-
-		if (c < 0xFFFF) {
-			snprintf(escape, sizeof(escape), "\\u%04X", c);
-			len += sink(escape, 6, writer);
-		} else {
-			snprintf(escape, sizeof(escape), "\\U%08X", c);
-			len += sink(escape, 10, writer);
-		}
+		i += size - 1;
 	}
 	return len;
 }
@@ -257,8 +379,7 @@ write_text(SerdWriter* writer, TextContext ctx,
 static size_t
 uri_sink(const void* buf, size_t len, void* stream)
 {
-	return write_text((SerdWriter*)stream, WRITE_URI,
-	                  (const uint8_t*)buf, len);
+	return write_uri((SerdWriter*)stream, (const uint8_t*)buf, len);
 }
 
 static void
@@ -313,6 +434,14 @@ typedef enum {
 	FIELD_OBJECT
 } Field;
 
+static bool
+is_inline_start(const SerdWriter* writer, Field field, SerdStatementFlags flags)
+{
+	return (writer->syntax != SERD_NTRIPLES &&
+	        ((field == FIELD_SUBJECT && (flags & SERD_ANON_S_BEGIN)) ||
+	         (field == FIELD_OBJECT &&  (flags & SERD_ANON_O_BEGIN))));
+}
+
 static bool
 write_node(SerdWriter*        writer,
            const SerdNode*    node,
@@ -322,13 +451,12 @@ write_node(SerdWriter*        writer,
            SerdStatementFlags flags)
 {
 	SerdChunk uri_prefix;
-	SerdChunk uri_suffix;
+	SerdNode  prefix;
+	SerdChunk suffix;
 	bool      has_scheme;
 	switch (node->type) {
 	case SERD_BLANK:
-		if (writer->syntax != SERD_NTRIPLES
-		    && ((field == FIELD_SUBJECT && (flags & SERD_ANON_S_BEGIN))
-		        || (field == FIELD_OBJECT && (flags & SERD_ANON_O_BEGIN)))) {
+		if (is_inline_start(writer, field, flags)) {
 			++writer->indent;
 			write_sep(writer, SEP_ANON_BEGIN);
 		} else if (writer->syntax != SERD_NTRIPLES
@@ -363,18 +491,27 @@ write_node(SerdWriter*        writer,
 	case SERD_CURIE:
 		switch (writer->syntax) {
 		case SERD_NTRIPLES:
-			if (serd_env_expand(writer->env, node, &uri_prefix, &uri_suffix)) {
+			if (serd_env_expand(writer->env, node, &uri_prefix, &suffix)) {
 				w_err(writer, SERD_ERR_BAD_CURIE,
 				      "undefined namespace prefix `%s'\n", node->buf);
 				return false;
 			}
 			sink("<", 1, writer);
-			write_text(writer, WRITE_URI, uri_prefix.buf, uri_prefix.len);
-			write_text(writer, WRITE_URI, uri_suffix.buf, uri_suffix.len);
+			write_uri(writer, uri_prefix.buf, uri_prefix.len);
+			write_uri(writer, suffix.buf, suffix.len);
 			sink(">", 1, writer);
 			break;
 		case SERD_TURTLE:
-			sink(node->buf, node->n_bytes, writer);
+			if (is_inline_start(writer, field, flags)) {
+				++writer->indent;
+				write_sep(writer, SEP_ANON_BEGIN);
+				sink("== ", 3, writer);
+			}
+			write_lname(writer, node->buf, node->n_bytes);
+			if (is_inline_start(writer, field, flags)) {
+				sink(" ;", 2, writer);
+				write_newline(writer);
+			}
 		}
 		break;
 	case SERD_LITERAL:
@@ -382,10 +519,18 @@ write_node(SerdWriter*        writer,
 			const char* type_uri = (const char*)datatype->buf;
 			if (!strncmp(type_uri, NS_XSD, sizeof(NS_XSD) - 1) && (
 				    !strcmp(type_uri + sizeof(NS_XSD) - 1, "boolean") ||
-				    !strcmp(type_uri + sizeof(NS_XSD) - 1, "decimal") ||
 				    !strcmp(type_uri + sizeof(NS_XSD) - 1, "integer"))) {
 				sink(node->buf, node->n_bytes, writer);
 				break;
+			} else if (!strcmp(type_uri + sizeof(NS_XSD) - 1, "decimal") &&
+			           strchr((const char*)node->buf, '.') &&
+			           node->buf[node->n_bytes - 1] != '.') {
+				/* xsd:decimal literals without trailing digits, e.g. "5.", can
+				   not be written bare in Turtle.  We could add a 0 which is
+				   prettier, but changes the text and breaks round tripping.
+				*/
+				sink(node->buf, node->n_bytes, writer);
+				break;
 			}
 		}
 		if (writer->syntax != SERD_NTRIPLES
@@ -407,6 +552,11 @@ write_node(SerdWriter*        writer,
 		}
 		break;
 	case SERD_URI:
+		if (is_inline_start(writer, field, flags)) {
+			++writer->indent;
+			write_sep(writer, SEP_ANON_BEGIN);
+			sink("== ", 3, writer);
+		}
 		has_scheme = serd_uri_string_has_scheme(node->buf);
 		if (field == FIELD_PREDICATE && (writer->syntax == SERD_TURTLE)
 		    && !strcmp((const char*)node->buf, NS_RDF "type")) {
@@ -416,15 +566,12 @@ write_node(SerdWriter*        writer,
 		           && !strcmp((const char*)node->buf, NS_RDF "nil")) {
 			sink("()", 2, writer);
 			break;
-		} else if (has_scheme && (writer->style & SERD_STYLE_CURIED)) {
-			SerdNode  prefix;
-			SerdChunk suffix;
-			if (serd_env_qualify(writer->env, node, &prefix, &suffix)) {
-				write_text(writer, WRITE_URI, prefix.buf, prefix.n_bytes);
-				sink(":", 1, writer);
-				write_text(writer, WRITE_URI, suffix.buf, suffix.len);
-				break;
-			}
+		} else if (has_scheme && (writer->style & SERD_STYLE_CURIED) &&
+		           serd_env_qualify(writer->env, node, &prefix, &suffix)) {
+			write_uri(writer, prefix.buf, prefix.n_bytes);
+			sink(":", 1, writer);
+			write_uri(writer, suffix.buf, suffix.len);
+			break;
 		}
 		sink("<", 1, writer);
 		if (writer->style & SERD_STYLE_RESOLVED) {
@@ -442,9 +589,13 @@ write_node(SerdWriter*        writer,
 					&uri, &writer->base_uri, root, uri_sink, writer);
 			}
 		} else {
-			write_text(writer, WRITE_URI, node->buf, node->n_bytes);
+			write_uri(writer, node->buf, node->n_bytes);
 		}
 		sink(">", 1, writer);
+		if (is_inline_start(writer, field, flags)) {
+			sink(" ;", 2, writer);
+			write_newline(writer);
+		}
 	default:
 		break;
 	}
@@ -502,15 +653,18 @@ serd_writer_write_statement(SerdWriter*        writer,
 		return SERD_ERR_BAD_ARG;
 	}
 
+#define TRY(write_result) \
+	if (!write_result) { \
+		return SERD_ERR_UNKNOWN; \
+	}
+
 	switch (writer->syntax) {
 	case SERD_NTRIPLES:
-		write_node(writer, subject, NULL, NULL, FIELD_SUBJECT, flags);
+		TRY(write_node(writer, subject, NULL, NULL, FIELD_SUBJECT, flags));
 		sink(" ", 1, writer);
-		write_node(writer, predicate, NULL, NULL, FIELD_PREDICATE, flags);
+		TRY(write_node(writer, predicate, NULL, NULL, FIELD_PREDICATE, flags));
 		sink(" ", 1, writer);
-		if (!write_node(writer, object, datatype, lang, FIELD_OBJECT, flags)) {
-			return SERD_ERR_UNKNOWN;
-		}
+		TRY(write_node(writer, object, datatype, lang, FIELD_OBJECT, flags));
 		sink(" .\n", 3, writer);
 		return SERD_SUCCESS;
 	default:
@@ -602,12 +756,11 @@ serd_writer_end_anon(SerdWriter*     writer,
 	if (writer->syntax == SERD_NTRIPLES) {
 		return SERD_SUCCESS;
 	}
-	if (serd_stack_is_empty(&writer->anon_stack)) {
+	if (serd_stack_is_empty(&writer->anon_stack) || writer->indent == 0) {
 		w_err(writer, SERD_ERR_UNKNOWN,
 		      "unexpected end of anonymous node\n");
 		return SERD_ERR_UNKNOWN;
 	}
-	assert(writer->indent > 0);
 	--writer->indent;
 	write_sep(writer, SEP_ANON_END);
 	reset_context(writer, true);
@@ -750,7 +903,7 @@ serd_writer_set_prefix(SerdWriter*     writer,
 			sink("@prefix ", 8, writer);
 			sink(name->buf, name->n_bytes, writer);
 			sink(": <", 3, writer);
-			write_text(writer, WRITE_URI, uri->buf, uri->n_bytes);
+			write_uri(writer, uri->buf, uri->n_bytes);
 			sink("> .\n", 4, writer);
 		}
 		writer->indent = 0;
