@@ -1,5 +1,5 @@
 /*
-  Copyright 2011-2012 David Robillard <http://drobilla.net>
+  Copyright 2011-2014 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -25,9 +25,11 @@
 
 #define USTR(s) ((const uint8_t*)(s))
 
-#ifdef _WIN32
+#ifndef INFINITY
 #    define INFINITY (DBL_MAX + DBL_MAX)
-#    define NAN      (INFINITY - INFINITY)
+#endif
+#ifndef NAN
+#    define NAN (INFINITY - INFINITY)
 #endif
 
 static int
@@ -45,7 +47,7 @@ static bool
 test_strtod(double dbl, double max_delta)
 {
 	char buf[1024];
-	snprintf(buf, sizeof(buf), "%lf", dbl);
+	snprintf(buf, sizeof(buf), "%f", dbl);
 
 	char* endptr = NULL;
 	const double out = serd_strtod(buf, &endptr);
@@ -100,11 +102,11 @@ main(void)
 	}
 
 	const double expt_test_nums[] = {
-		2.0E18, -5e19, +8e20, 2e+34, -5e-5, 8e0, 9e-0, 2e+0
+		2.0E18, -5e19, +8e20, 2e+24, -5e-5, 8e0, 9e-0, 2e+0
 	};
 
 	const char* expt_test_strs[] = {
-		"02e18", "-5e019", "+8e20", "2E+34", "-5E-5", "8E0", "9e-0", " 2e+0"
+		"02e18", "-5e019", "+8e20", "2E+24", "-5E-5", "8E0", "9e-0", " 2e+0"
 	};
 
 	for (unsigned i = 0; i < sizeof(expt_test_nums) / sizeof(double); ++i) {
@@ -378,6 +380,11 @@ main(void)
 		return failure("%s != %s\n", lhs.buf, lhs.buf);
 	}
 
+	SerdNode null_copy = serd_node_copy(&SERD_NODE_NULL);
+	if (!serd_node_equals(&SERD_NODE_NULL, &null_copy)) {
+		return failure("copy of null node != null node\n");
+	}
+
 	// Test serd_node_from_string
 
 	SerdNode node = serd_node_from_string(SERD_LITERAL, (const uint8_t*)"hello\"");
@@ -394,15 +401,23 @@ main(void)
 
 	// Test serd_node_new_uri_from_string
 
+	SerdNode nonsense = serd_node_new_uri_from_string(NULL, NULL, NULL);
+	if (nonsense.type != SERD_NOTHING) {
+		return failure("Successfully created NULL URI\n");
+	}
+
 	SerdURI base_uri;
 	SerdNode base = serd_node_new_uri_from_string(USTR("http://example.org/"),
 	                                              NULL, &base_uri);
 	SerdNode nil = serd_node_new_uri_from_string(NULL, &base_uri, NULL);
-	if (nil.type != SERD_URI || strcmp((const char*)nil.buf, (const char*)base.buf)) {
+	SerdNode nil2 = serd_node_new_uri_from_string(USTR(""), &base_uri, NULL);
+	if (nil.type != SERD_URI || strcmp((const char*)nil.buf, (const char*)base.buf) ||
+	    nil2.type != SERD_URI || strcmp((const char*)nil2.buf, (const char*)base.buf)) {
 		return failure("URI %s != base %s\n", nil.buf, base.buf);
 	}
 	serd_node_free(&base);
 	serd_node_free(&nil);
+	serd_node_free(&nil2);
 
 	// Test SerdEnv
 
@@ -412,10 +427,18 @@ main(void)
 	SerdEnv* env = serd_env_new(NULL);
 	serd_env_set_prefix_from_strings(env, USTR("eg.2"), USTR("http://example.org/"));
 
+	if (!serd_env_set_base_uri(env, NULL)) {
+		return failure("Successfully set NULL base URI\n");
+	}
+
 	if (!serd_env_set_base_uri(env, &node)) {
 		return failure("Set base URI to %s\n", node.buf);
 	}
 
+	if (!serd_node_equals(serd_env_get_base_uri(env, NULL), &node)) {
+		return failure("Base URI mismatch\n");
+	}
+
 	SerdChunk prefix, suffix;
 	if (!serd_env_expand(env, &b, &prefix, &suffix)) {
 		return failure("Expanded invalid curie %s\n", b.buf);
@@ -602,7 +625,10 @@ main(void)
 		return failure("Apparently read an http URI\n");
 	}
 	if (!serd_reader_read_file(reader, USTR("file:///better/not/exist"))) {
-		return failure("Apprently read a non-existent file\n");
+		return failure("Apparently read a non-existent file\n");
+	}
+	if (!serd_reader_read_file(reader, USTR("file://"))) {
+		return failure("Apparently read a file with no path\n");
 	}
 	SerdStatus st = serd_reader_read_file(reader, USTR(path));
 	if (st) {
